import pygame


class Coords(object):
    """
    A type for managing texture coordinates.
    Use * and / for scaling, + and - for translating.
    """
    def __init__(self, lrbt=(0.0,1.0,0.0,1.0)):
        self.lrbt = list(lrbt)
        
    def __repr__(self):
        return repr((self[0],self[1],self[2],self[3]))
        
    def __imul__(self, scalar):
        self.lrbt[:] = [i * scalar for i in self.lrbt]
        return self
        
    def __idiv__(self, scalar):
        self.lrbt[:] = [i / scalar for i in self.lrgb]
        return self
    
    def __iadd__(self, vec2):
        self.lrbt[0] += vec2[0]
        self.lrbt[1] += vec2[0]
        self.lrbt[2] += vec2[1]
        self.lrbt[3] += vec2[1]
        return self
        
    def __isub__(self, vec2):
        self.lrbt[0] -= vec2[0]
        self.lrbt[1] -= vec2[0]
        self.lrbt[2] -= vec2[1]
        self.lrbt[3] -= vec2[1]
        return self
    
    def __mul__(self, scalar):
        return Coords([i * scalar for i in self.lrbt])
        
    def __div__(self, scalar):
        return Coords([i / scalar for i in self.lrbt])
    
    def __add__(self, vec2):
        return Coords((self.lrbt[0]+vec2[0],self.lrbt[1]+vec2[0],self.lrbt[2]+vec2[1],self.lrbt[3]+vec2[1]))

    def __sub__(self, vec2):
        return Coords((self.lrbt[0]-vec2[0],self.lrbt[1]-vec2[0],self.lrbt[2]-vec2[1],self.lrbt[3]-vec2[1]))
        
    def __getitem__(self, i):
        if i == 0:
            return self.lrbt[0], self.lrbt[2]
        elif i == 1:
            return self.lrbt[0], self.lrbt[3]
        elif i == 2:
            return self.lrbt[1], self.lrbt[3]
        elif i == 3:
            return self.lrbt[1], self.lrbt[2]
        else:
            raise IndexError
        
        
class PackNode(object):
    """
    Creates an area which can recursively pack images into itself.
    """
    def __init__(self, area):
        #if tuple contains two elements, assume they are width and height, and origin is (0,0)
        if len(area) == 2:
            area = (0,0,area[0],area[1])
        self.area = area

    def __repr__(self):
        return "<%s %s>" % (self.__class__.__name__, str(self.area))

    def get_width(self):
        return self.area[2] - self.area[0]
    width = property(fget=get_width)

    def get_height(self):
        return self.area[3] - self.area[1]
    height = property(fget=get_height)

    def insert(self, area):
        if hasattr(self, 'child'):
            a = self.child[0].insert(area)
            if a is None: return self.child[1].insert(area)
            return a

        area = PackNode(area)
        if area.width <= self.width and area.height <= self.height:
            self.child = [None,None]
            self.child[0] = PackNode((self.area[0]+area.width, self.area[1], self.area[2], self.area[1] + area.height))
            self.child[1] = PackNode((self.area[0], self.area[1]+area.height, self.area[2], self.area[3]))
            return PackNode((self.area[0], self.area[1], self.area[0]+area.width, self.area[1]+area.height))


class Pack(object):
    def __init__(self, size=(512,512)):
        self.tree = PackNode(size)
        self.image = pygame.Surface(size, pygame.SRCALPHA, 32)
        self.wr = 1.0 / float(size[0])
        self.hr = 1.0 / float(size[1])
        
    def pack(self, img):
        wr = self.wr
        hr = self.hr
        if not isinstance(img, pygame.Surface):
            img = pygame.image.load(img).convert(32, pygame.SRCALPHA)
            img = img.convert_alpha(img)
        uv = self.tree.insert(img.get_size())
        if uv is None: raise ValueError('Pack size too small.')
        area = tuple(uv.area)
        self.image.blit(img, area)
        uv = area[0] * wr, area[1] * hr, area[2] * wr, area[3] * hr
        return (uv[0],uv[1]),(uv[0], uv[3]), (uv[2],uv[3]), (uv[2], uv[1])


